// Moves enemy to the player.
class ObjectMover : Inventory
{
	Actor to;
	int dist;
	
	override void DoEffect()
	{
		if(!Owner) return;
		
		vector3 delta = ( to.pos.x - Owner.pos.x, to.pos.y - Owner.pos.y, to.pos.z - Owner.pos.z );
		double angleto = Owner.AngleTo(to);
		double pitchto = VectorAngle( sqrt(delta.y * delta.y + delta.x * delta.x), delta.z );
		to.A_SetPitch(pitchto,SPF_INTERPOLATE);
		to.A_SetAngle(angleto+180,SPF_INTERPOLATE);
		float zpush = 10.0 * sin(pitchto);
		if (to.Distance3D(Owner) > dist)
		{
			vector3 neworigin = Owner.pos;
			neworigin.x += cos(angleto) * 20;
			neworigin.y += sin(angleto) * 20;
			neworigin.z += zpush;
			Owner.SetOrigin(neworigin,true);
		}
		else
		{
			RemoveInventory(self);
		}
		super.DoEffect();
	}
}

// Displays the stagger graphic.
class ShadedActor : Inventory
{
	Actor act;
	
	Default
	{
		+NOINTERACTION
		+MOVEWITHSECTOR
	}
	override void BeginPlay()
	{
		A_SetRenderStyle(1.0,STYLE_AddShaded);
		Color gloryshade = Color(255,128,0);
		SetShade(gloryshade);
		super.BeginPlay();
	}
	override void Tick(void)
	{
		if(!act || act.health <= 0) GoAwayAndDie();
		if(act)
		{
			if(radius != act.radius || height != act.height) A_SetSize(0.2, 0.2);
			if(scale.x != act.scale.x || scale.y != act.scale.y) A_SetScale(act.scale.x,act.scale.y);
			Sprite = act.sprite;
			frame = act.frame;
			angle = act.angle;
			vector3 correctedpos = (act.pos.x+cos(act.angle)*5,act.pos.y+sin(act.angle)*5,act.pos.z);
			SetOrigin(correctedpos,true);
			Spawn("ShadedActor_Light",act.pos);
		}
		super.Tick();
	}
}
class ShadedActor_Light : Actor
{
	Default
	{
		+NOINTERACTION
		+MOVEWITHSECTOR
	}
	States
	{
		Spawn:
			TNT1 A 1;
		SpawnDie:
			TNT1 A 1;
		stop;
	}
}

// Staggers enemies, utilizes graphic.
class IStagger : Inventory
{
	int ptics;
	int livetics;
	ShadedActor ashader;
	
    Default
    {
        Inventory.MaxAmount 1;
    }
	
	void ToggleShade()
	{
		if(!Owner) return;
		if(!ashader)
		{
			ashader = ShadedActor(Spawn("ShadedActor",Owner.pos));
			ashader.act = Owner;
		} 
		else if(ashader)
		{
			ashader.GoAwayAndDie();
			ashader = null;
		}
	}

    override void DoEffect()
    {
		if(!Owner || Owner.Health <= 0)
		{
			DepleteOrDestroy();
			return;
		}

		if(!ptics) ptics = Owner.tics;
		Owner.tics = -1;
		
		if(!livetics) ToggleShade();
		if( livetics >= sv_staggerlength*0.6 && !(level.maptime%10) ) ToggleShade(); 
		livetics ++;
		
		if(livetics > sv_staggerlength) DepleteOrDestroy();
        super.DoEffect();
    }
    
    override void DepleteOrDestroy()
    {
		if(ashader) ashader.GoAwayAndDie();
        if(Owner) 
        {
			if(Owner.health > 0) Owner.A_SetHealth(Owner.health + (1.0-(Owner.health*sv_staggerhealth))/2 );
            Owner.tics = ptics;
            Owner.RemoveInventory(self);
        }
    }
}

// Health drops
class SuperHealthBonus : CustomInventory
{
	int livetics;
	Actor plr;
	Default
	{		
		-NOTELEPORT
		-NOLIFTDROP
		+NOBLOCKMONST
		+NOTONAUTOMAP
		-COUNTITEM
		+INVENTORY.ALWAYSPICKUP
		Inventory.PickupMessage "Glory kill bonus";
		Inventory.PickupSound "ABonus";
		Scale 0.1;
		Radius 3;
		Height 3;
	}
	
	States 
	{
		Spawn:
			TNT1 A 0;
			HEIC A -1 Bright;
			stop;
		Pickup:
			TNT1 A 0 A_GiveInventory ("Healthbonus", 5);
			stop;
	}
	
	override String PickupMessage()
	{
		return "Glory kill bonus";
	}
	
	override void Tick()
	{
		if(pos.z == floorz) bNOGRAVITY = true;
		livetics++;
		if(plr && Distance2D(plr) > radius && livetics > 10)
		{
			vector3 delta = ( plr.pos.x - pos.x, plr.pos.y - pos.y, (plr.pos.z+plr.player.viewheight/2) - pos.z );
			double angleto = AngleTo(plr);
			double pitchto = VectorAngle( sqrt(delta.y * delta.y + delta.x * delta.x), delta.z );
			float zpush = 10.0 * sin(pitchto);
			vector3 neworigin = pos;
			neworigin.x += cos(angleto) * 10;
			neworigin.y += sin(angleto) * 10;
			neworigin.z += zpush;
			SetOrigin(neworigin,true);
		}
		super.Tick();
	}
}

//Ammo drops
class SuperBackpack : CustomInventory
{
	int livetics;
	Actor plr;
	Default
	{
		-NOTELEPORT
		-NOLIFTDROP
		+NOBLOCKMONST
		+NOTONAUTOMAP
		-COUNTITEM
		+INVENTORY.ALWAYSPICKUP
		Inventory.PickupSound "ABonus";
		Scale 0.1;
		Radius 3;
		Height 3;
	}
	
	States 
	{
		Spawn:
			TNT1 A 0;
			AMIC A -1 Bright;
			stop;
		Pickup:
			TNT1 A 0 A_GiveInventory ("Clip", 5);
			TNT1 A 0 A_GiveInventory ("Shell", 3);
			TNT1 A 0 A_GiveInventory ("RocketAmmo", 2);
			TNT1 A 0 A_GiveInventory ("Cell", 5);
			TNT1 A 0 A_GiveInventory ("Fuel", 7);
			stop;
	}
	
	override String PickupMessage()
	{
		return "";
	}
	
	override void Tick()
	{
		if(pos.z == floorz) bNOGRAVITY = true;
		livetics++;
		if(plr && Distance2D(plr) > radius && livetics > 10)
		{
			vector3 delta = ( plr.pos.x - pos.x, plr.pos.y - pos.y, (plr.pos.z+plr.player.viewheight/2) - pos.z );
			double angleto = AngleTo(plr);
			double pitchto = VectorAngle( sqrt(delta.y * delta.y + delta.x * delta.x), delta.z );
			float zpush = 10.0 * sin(pitchto);
			vector3 neworigin = pos;
			neworigin.x += cos(angleto) * 10;
			neworigin.y += sin(angleto) * 10;
			neworigin.z += zpush;
			SetOrigin(neworigin,true);
		}
		super.Tick();
	}
}

//Armor drops
class SuperArmorBonus : CustomInventory
{
	int livetics;
	Actor plr;
	Default
	{
		-NOTELEPORT
		-NOLIFTDROP
		+NOBLOCKMONST
		+NOTONAUTOMAP
		-COUNTITEM
		+INVENTORY.ALWAYSPICKUP
		Inventory.PickupSound "ABonus";
		Scale 0.1;
		Radius 3;
		Height 3;
	}
	
	States 
	{
		Spawn:
			TNT1 A 0;
			ARIC A -1 Bright;
			stop;
		Pickup:
			TNT1 A 0 A_GiveInventory ("ArmorBonus", 5);
			TNT1 A 0 A_GiveInventory ("ArmorBonus", 5);
			TNT1 A 0 A_GiveInventory ("ArmorBonus", 5);
			TNT1 A 0 A_GiveInventory ("ArmorBonus", 5);
			TNT1 A 0 A_GiveInventory ("ArmorBonus", 5);
			stop;
	}
	
	override String PickupMessage()
	{
		return "";
	}
	
	override void Tick()
	{
		if(pos.z == floorz) bNOGRAVITY = true;
		livetics++;
		if(plr && Distance2D(plr) > radius && livetics > 10)
		{
			vector3 delta = ( plr.pos.x - pos.x, plr.pos.y - pos.y, (plr.pos.z+plr.player.viewheight/2) - pos.z );
			double angleto = AngleTo(plr);
			double pitchto = VectorAngle( sqrt(delta.y * delta.y + delta.x * delta.x), delta.z );
			float zpush = 10.0 * sin(pitchto);
			vector3 neworigin = pos;
			neworigin.x += cos(angleto) * 10;
			neworigin.y += sin(angleto) * 10;
			neworigin.z += zpush;
			SetOrigin(neworigin,true);
		}
		super.Tick();
	}
}

// Hacky :P
class kill_doppelganger : inventory
{
	int wait;
	override void DoEffect()
	{	
		wait++;
		if(wait>=2 && Owner.master) 
		{
			Owner.bNODAMAGE = false;
			Owner.master.Destroy();
			Owner.master = NULL;
			Owner.RemoveInventory(self);
		}
	}
}

